@@ -29,16 +29,15 @@ int2col <- function(x) {
 col2int <- function(x) {
   if (is.null(x)) return(NULL)
 
-  if (is.numeric(x) || is.integer(x) || is.factor(x))
+  if (is.numeric(x) || is.factor(x))
     return(as.integer(x))
 
   if (!is.character(x)) {
     stop("x must be character")
-
-    if (anyNA(x))
-      stop("x must be a valid character")
   }
 
+  if (anyNA(x)) stop("x contains NA")
+
   if (any(grepl(":", x))) {
     # loop through x until all ":" are replaced with integer sequences
     while (any(grepl(":", x))) {
@@ -108,5 +107,14 @@ calc_col_width <- function(base_font, col_width) {
   # to the expected widths
   widths <- trunc((as.numeric(col_width) * mdw + 5) / mdw * 256) / 256
   widths <- round(widths, 3)
+
+  if (any(sel <- widths > getOption("openxlsx2.maxWidth", 250))) {
+    widths[sel] <- getOption("openxlsx2.maxWidth", 250)
+  }
+
+  if (any(sel <- widths <= getOption("openxlsx2.minWidth", 0))) {
+    widths[sel] <- getOption("openxlsx2.minWidth", 1)
+  }
+
   widths
 }
