@@ -77,10 +77,10 @@ test_that("`wb_dims()` works/errors as expected with unnamed arguments", {
   expect_error(wb_dims(mtcars), "Supplying a single unnamed argument")
 
 
-  expect_error(wb_dims(rows = c(1, 3, 4), cols = c(1, 4)), "You must supply positive, consecutive values to `cols`")
+  expect_error(wb_dims(rows = c(1), cols = c(-1)), "You must supply positive values to `cols`")
+  expect_error(wb_dims(rows = c(-1), cols = c(1)), "You must supply positive values to `rows`")
 
   expect_error(wb_dims(1, 2, 3))
-  expect_error(wb_dims(x = mtcars, select = c("bad", "col_names")), "accepts a single")
 })
 
 test_that("`wb_dims()` errors when providing unsupported arguments", {
@@ -123,8 +123,6 @@ test_that("wb_dims() works when not supplying `x`.", {
   expect_error(wb_dims(3, 0))
   expect_error(wb_dims(1, 1, col_names = TRUE))
   expect_error(wb_dims(1, 1, row_names = FALSE), "`row_names`")
-  expect_error(wb_dims(2, 10, select = "col_names"), "Can't supply `select` when `x` is absent")
-
 })
 
 test_that("`wb_dims()` can select content in a nice fashion with `x`", {
@@ -171,6 +169,46 @@ test_that("`wb_dims()` can select content in a nice fashion with `x`", {
   expect_equal(wb_dims_cars(rows = 1:5, select = "x"), dims_row1_to_5_and_names)
 })
 
+test_that("wb_dims can select multiple columns and rows", {
+
+  exp <- "B2:B33"
+  got <- wb_dims(x = mtcars, cols = c("cyl"))
+  expect_equal(exp, got)
+
+  exp <- "B2:B33,J2:J33"
+  got <- wb_dims(x = mtcars, cols = c("cyl", "gear"))
+  expect_equal(exp, got)
+
+  exp <- "B1,J1"
+  got <- wb_dims(x = mtcars, cols = c("cyl", "gear"), select = "col_names")
+  expect_equal(exp, got)
+
+  exp <- "A1:K33"
+  got <- wb_dims(x = mtcars)
+  expect_equal(exp, got)
+
+  exp <- "B2:B33"
+  got <- wb_dims(x = mtcars, cols = c(2))
+  expect_equal(exp, got)
+
+  exp <- "B2:B33,D2:D33"
+  got <- wb_dims(x = mtcars, cols = c(2, 4))
+  expect_equal(exp, got)
+
+  exp <- "A5:K5"
+  got <- wb_dims(x = mtcars, rows = c(4))
+  expect_equal(exp, got)
+
+  exp <- "A3:K3,A5:K5,A6:K6"
+  got <- wb_dims(x = mtcars, rows = c(2, 4:5))
+  expect_equal(exp, got)
+
+  exp <- "B3:B3,D3:D3,B5:B5,D5:D5,B6:B6,D6:D6"
+  got <- wb_dims(x = mtcars, cols = c(2, 4), rows = c(2, 4:5))
+  expect_equal(exp, got)
+
+})
+
 test_that("`wb_dims()` works for a matrix without column names", {
   mt <- matrix(c(1, 2))
   expect_equal(wb_dims(x = mt), "A1:A3")
@@ -199,7 +237,7 @@ test_that("`wb_dims()` works when Supplying an object `x`.", {
   expect_equal(wb_dims(x = mtcars, rows = 5:10, from_col = "C"), "C6:M11")
   # Write without column names on top
 
-  expect_error(wb_dims(x = mtcars, cols = 0, from_col = "C"), "supply positive.+ values to `cols`")
+  expect_error(wb_dims(x = mtcars, cols = 0, from_col = "C"), "You must supply positive values to `cols`")
 
   # select rows and columns work
   expect_equal(wb_dims(x = mtcars, rows = 2:10, cols = "cyl"), "B3:B11")
@@ -224,10 +262,7 @@ test_that("`wb_dims()` works when Supplying an object `x`.", {
   expect_error(wb_dims(cols = "hp"), "Supplying a single argument")
   # using non-existing character column doesn't work
   expect_error(wb_dims(x = mtcars, cols = "A"), "`cols` must be an integer or an existing column name of `x`.")
-  expect_error(
-    wb_dims(x = mtcars, cols = c("hp", "vs")),
-    regexp = "Supplying multiple column names is not supported"
-  )
+  expect_equal(wb_dims(x = mtcars, cols = c("hp", "vs")), "D2:D33,H2:H33")
   expect_error(expect_warning(wb_dims(x = mtcars, rows = "hp")), "[Uu]se `cols` instead.")
   # Access only row / col name
   expect_no_message(wb_dims(x = mtcars, select = "col_names"))
@@ -272,6 +307,17 @@ test_that("`wb_dims()` handles row_names = TRUE consistenly.", {
   # Style row names of an object
 })
 
+test_that("wb_dims() errors clearly with bad `select`.", {
+  expect_error(wb_dims(x = mtcars, select = c("bad", "col_names")), "accepts a single")
+  expect_error(wb_dims(2, 10, select = "col_names"), "Can't supply `select` when `x` is absent")
+
+  expect_error(wb_dims(x = mtcars, col_names = FALSE, select = "col_names"), "col_names = TRUE")
+  expect_error(wb_dims(x = mtcars, col_names = FALSE, select = "row_names"), "row_names = TRUE")
+  expect_error(wb_dims(x = mtcars, row_names = FALSE, select = "row_names"), "row_names = TRUE")
+  # row_names = FALSE is the default
+  expect_error(wb_dims(x = mtcars, select = "row_names"), "row_names = TRUE")
+})
+
 test_that("wb_dims() handles empty data frames", {
   expect_equal(wb_dims(x = data.frame(), from_col = "B"), "B1")
 })
@@ -515,3 +561,19 @@ test_that("wb_dims(from_dims) positioning works", {
   expect_equal(exp, got)
 
 })
+
+test_that("dims are quick", {
+
+  ddims <- dims_to_dataframe(
+    "A1:D5000",
+    fill = TRUE
+  )
+
+  edims <- dims_to_dataframe(
+    "A1:A5000,B1:B5000,C1:C5000,D1:D5000",
+    fill = TRUE
+  )
+
+  expect_equal(ddims, edims)
+
+})
